include<tile.scad>;

LX15D_dim = [ 44.17, 22.95, 26 ];
LX15D_middle_t = 1.5;  // height of the middle space
LX15D_middle_a = 3.0;  // side space
LX15D_axis_offset = 10.66;
LX15D_horn_d = 20;
LX15D_horn_w = 38.6;
LX15D_screw_x = 41.2;
LX15D_screw_y = 18;
LX15D_horn_screw_d = 3;
LX15D_screw_clearance_d = 7;
LX15D_body_screw_d = 2;
LX15D_horn_clearance_d = 8;
LX15D_clearance_r = sqrt(pow(LX15D_dim[1] / 2, 2) + pow(LX15D_axis_offset, 2));

module LX15D() color("gray") {
    dim = LX15D_dim;

    translate([ dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
        cube(dim, center = true);
        difference() {
            cube(dim + [ -LX15D_middle_a, -LX15D_middle_a, LX15D_middle_t * 2 ],
                 center = true);
            for (i = [ -0.5, 0.5 ], j = [ -0.5, 0.5 ])
                translate([ dim[0] * i, dim[1] * j, 0 ])
                    cylinder(d = 13, h = 100, center = true);
        }
    }

    cylinder(d = LX15D_horn_d, h = LX15D_horn_w, center = true);
}

module LX15D_collision() {
    dim = LX15D_dim;

    translate([ dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
        cube(dim + [ 0, 0, LX15D_middle_t * 2 ], center = true);
    }

    cube([ LX15D_horn_d, LX15D_horn_d, LX15D_horn_w ], center = true);
}

module LX15D_clearance() {
    cylinder(r = LX15D_clearance_r, h = LX15D_horn_w, center = true);
}

module LX15D_screw_plane() { translate([ 0, 0, LX15D_dim[2] / 2 ]) children(); }

module LX15D_bottom_screw_pos()
    translate([ LX15D_dim[0] / 2 - LX15D_axis_offset, 0, 0 ]) {
    for (y = [ -0.5, 0.5 ])
        translate([ LX15D_screw_x / 2, y * LX15D_screw_y, LX15D_dim[2] / 2 ])
            children();
}

module LX15D_horn_screw_pos() {
    for (a = [0:90:360])
        rotate([ 0, 0, a ]) translate([ 14 / 2, 0, 0 ]) children();
}
module LX15D_side_plate(screw_d, holes, t) difference() {
    union() {
        LX15D_bottom_screw_pos() hull() {
            cylinder(d = LX15D_screw_clearance_d, h = t);
            translate([ t * 2, 0, 0 ])
                cylinder(d = LX15D_screw_clearance_d, h = t);
        }
        hull() for (coord = holes) {
            translate(coord + [ 0, 0, LX15D_dim[2] / 2 ])
                cylinder(d = screw_d + t * 2, h = t);
        }
    }
    for (coord = holes)
        translate(coord) cylinder(d = screw_d, h = 100, center = true);
    LX15D_bottom_screw_pos()
        cylinder(d = LX15D_body_screw_d, h = 100, center = true);
}
LX15D_U_h = 6;
LX15D_U_c = 0.2;
LX15D_U_offset =
    [ LX15D_dim[0] - LX15D_axis_offset + LX15D_U_c + LX15D_U_h, 0, 0 ];
module LX15D_U() render() difference() {
    a = 24;
    t = 2;
    union() {
        translate([ LX15D_dim[0] - LX15D_axis_offset + LX15D_U_c, 0, 0 ])
            difference() {
            translate([ LX15D_U_h / 2, 0, 0 ])
                cube([ LX15D_U_h, a, LX15D_dim[2] + t * 2 ], center = true);
            translate([ LX15D_U_h, 0, 0 ]) rotate([ 0, -90, 0 ])
                tile_neg(T24, LX15D_U_h);
        }

        for (m = [ 0, 1 ])
            mirror([ 0, 0, m ]) LX15D_bottom_screw_pos()
                hull() for (x = [ 0, LX15D_U_h / 2 ]) translate([ x, 0, 0 ])
                    cylinder(d = LX15D_screw_clearance_d, h = t);
    }
    LX15D_bottom_screw_pos()
        cylinder(d = LX15D_body_screw_d, h = 100, center = true);
}
LX15D_BU_h = 6;
LX15D_BU_screw_d = 3;
LX15D_BU_screw_h = 6;
LX15D_BU_screw_a = 10;
LX15D_BU_x_off = -LX15D_clearance_r - LX15D_BU_h / 2;
module LX15D_BU_plate() difference() {
    t = 3;
    hull() translate([ 0, 0, LX15D_horn_w / 2 ]) {
        LX15D_horn_screw_pos() cylinder(d = LX15D_horn_screw_d + t * 2, h = t);
        for (i = [ -0.5, 0.5 ])
            translate([ LX15D_BU_x_off, i * LX15D_BU_screw_a, 0 ])
                cylinder(d = LX15D_BU_screw_d + t * 2, h = t);
    }
    LX15D_horn_screw_pos()
        cylinder(d = LX15D_horn_screw_d, h = 100, center = true);
    for (i = [ -0.5, 0.5 ])
        translate([ LX15D_BU_x_off, i * LX15D_BU_screw_a, 0 ])
            cylinder(d = LX15D_BU_screw_d, h = 100, center = true);
}
module LX15D_BU_body() difference() {
    dim = [ LX15D_BU_h, LX15D_dim[1], LX15D_horn_w ];
    x_off = LX15D_BU_x_off;
    union() {
        translate([ x_off, 0, 0 ]) difference() {
            cube(dim, center = true);
            rotate([ 0, 90, 0 ]) tile_neg(T24, LX15D_BU_h, center = true);
        }
    }

    for (z = [ -dim[2] / 2, dim[2] / 2 ], i = [ -0.5, 0.5 ])
        translate([ x_off, i * LX15D_BU_screw_a, z ]) cylinder(
            d = LX15D_BU_screw_d, h = LX15D_BU_screw_h * 2, center = true);
}

module LX15D_BU() {
    LX15D_BU_body();
    for (m = [ 0, 1 ]) mirror([ 0, 0, m ]) LX15D_BU_plate();
}
