include<LX15D.scad>;
include<tile.scad>;

module tibia_servo_pos() rotate([ 90, 0, 0 ]) children();

t = 1.5;
screw_d = 2;
screw_t = 1.5;
screw_step = 5;
b = 8;
leg_i = 10;

slide_w = 5;
slide_h = screw_d + screw_t * 2;

tile_h = 6;

function screw_pos(i) = tile_h + t * 3 + i * screw_step;

module screw_positions(l) for (i = [0:(l - b / 2 - screw_pos(0) - screw_d) /
                                      screw_step],
                               alpha = [0:90:360]) rotate([ alpha, 0, 0 ])
    translate([ screw_pos(i), 0, 0 ]) children();

module tibia_tip(tipOffset) render() translate(LX15D_U_offset) difference() {
    l = tipOffset[0] - LX15D_U_offset[0];
    union() {
        translate([ l / 2, 0, 0 ]) for (alpha = [ 0, 90 ])
            rotate([ alpha, 0, 0 ])
                cube([ l - b, slide_w + screw_t * 2, b + slide_h * 2 ],
                     center = true);
        rotate([ 0, 90, 0 ]) hull() tile_pos(T24, tile_h);
        rotate([ 360 / 16, 0, 0 ]) {
            hull() {
                rotate([ 0, 90, 0 ]) cylinder(d = b, h = l - b / 2, $fn = 8);
                translate([ l, 0, 0 ]) rotate([ 0, -90, 0 ])
                    cylinder(d = 1, h = b, $fn = 8);
            }
        }
    }

    screw_positions(l) translate([ 0, b / 2 + screw_d / 2 + screw_t, 0 ])
        cylinder(d = screw_d, h = 100, center = true);

    translate([ l / 2 + tile_h, 0, 0 ]) for (alpha = [0:90:360])
        rotate([ alpha, 0, 0 ]) translate([ 0, 0, b / 2 + slide_h ])
            cube([ l, slide_w, slide_h * 2 ], center = true);
    rotate([ 0, 90, 0 ]) tile_neg(T24, tile_h);
}

function spring_f(i, m, r) =
    [[cos(i - 90), sin(i - 90)], [-sin(i - 90), cos(i - 90)]] *
    [ r + sign(sin(i * m)) * pow(abs(sin(i * m)), 0.2) * 2, 0 ];

leg_h = slide_w;
bot_d = 40;
module leg_bot_base() offset(t) offset(-t) {
    intersection() {
        difference() {
            circle(d = bot_d);
            circle(d = bot_d * 0.3, center = true);
        }
        rotate([ 0, 0, -45 ]) translate([ 1, 1, 0 ]) square(bot_d);
    }
}

module leg_bot() render() linear_extrude(height = t) {
    hex_d = 4;
    off_x = 7;
    off_y = 2;
    difference() {
        leg_bot_base();
        intersection() {
            offset(-1) leg_bot_base();
            for (x = [0:off_x:bot_d / 2], y = [-bot_d / 2:hex_d:bot_d / 2]) {
                translate([ x, y, 0 ]) circle(d = hex_d, $fn = 6);
                translate([ x + off_x / 2, y + off_y, 0 ])
                    circle(d = hex_d, $fn = 6);
            }
        }
    }
}

module leg(foot_alpha = 10) render() difference() {
    screw_slots = [ 1, 4 ];
    leg_base_dim =
        [ screw_step * abs(screw_slots[0] - screw_slots[1]), leg_h, t ];
    foot = [ 2, leg_h, 14 ];
    off = 6;
    spring_t = 0.7;
    spring_r = [ 10, 16 ];
    union() {
        for (i = screw_slots)
            translate([ -screw_step * i, 0, b / 2 + slide_h / 2 ])
                rotate([ 90, 0, 0 ]) cylinder(d = screw_d + screw_t * 2,
                                              h = slide_w, center = true);
        translate([
            -screw_step * (screw_slots[0] + screw_slots[1]) / 2, 0,
            b / 2 + leg_base_dim[2] / 2
        ]) cube(leg_base_dim, center = true);

        translate([ 0, 0, b / 2 ]) {
            render() rotate([ 90, 0, 0 ]) linear_extrude(
                height = leg_h, center = true) for (i = [-85:0.2:foot_alpha],
                                                    r = spring_r) hull() {
                translate(spring_f(i, 1.5 * r, r)) circle(d = spring_t);
                translate(spring_f(i - 1, 1.5 * r, r)) circle(d = spring_t);
            }
            rotate([ 0, foot_alpha, 0 ]) {
                translate([ 0, 0, foot[2] / 2 + off ])
                    cube(foot, center = true);
                rotate([ 180, -90, 0 ]) leg_bot();
            }
        }
    }
    for (i = screw_slots)
        translate([ -screw_step * i, 0, b / 2 + slide_h / 2 ])
            rotate([ 90, 0, 0 ]) cylinder(d = screw_d, h = 100, center = true);
}

$fa = 1;
$fs = 1;

module tibia(tipOffset) {
    tibia_servo_pos() LX15D();
    color("yellow") {
        tibia_servo_pos() LX15D_U();
        tibia_tip(tipOffset);

        for (beta = [0:90:360])
            rotate([ beta, 0, 0 ])
                translate(LX15D_U_offset + [ screw_pos(leg_i), 0, 0 ]) leg();
    }
}
module tibia_leg_col(tibiaOffset) { hull() leg(foot_alpha = -30); }

module tibia_collision(tipOffset) {
    dim = [ 64, 40, 26 ];
    dim2 = [ tipOffset[0], b, b ];
    off = [ 20, 0, 0 ];

    translate(off) cube(dim, center = true);

    translate([ dim2[0] / 2, 0, 0 ]) cube(dim2, center = true);

    for (beta = [0:90:360]) rotate([ beta, 0, 0 ]) tibia_leg_col(tipOffset);
}

tibia([ 102, 0, 0 ]);
